# coding: utf-8
from django.contrib.auth.models import User as AuthUser
from django.contrib.auth.backends import ModelBackend
from rest_framework import serializers
from rest_framework.authtoken.models import Token
from . import models


class CheckToken(serializers.Serializer):
    token = serializers.CharField()

    def validate_token(self, value):
        try:
            return Token.objects.get(key=value)
        except Token.DoesNotExist:
            raise serializers.ValidationError('Token not found')


class CheckPerm(serializers.Serializer):
    user = serializers.CharField()
    perm = serializers.CharField(max_length=50)

    def validate_user(self, value):
        try:
            return AuthUser.objects.get(username=value)
        except AuthUser.DoesNotExist:
            raise serializers.ValidationError('User with username "{0}" not found'.format(value))

    def validate_perm(self, value):
        try:
            return models.Permission.objects.get(code=value)
        except models.Permission.DoesNotExist:
            raise serializers.ValidationError('Permission with code "{0}" not found'.format(value))

    def validate(self, attrs):
        user = attrs.get('user')
        perm = attrs.get('perm')
        if user and perm:
            user_qs = models.UserPermission.objects.filter(user=user, permission=perm)

            group_qs = models.GroupPermission.objects.filter(
                group__in=user.groups.all(),
                permission=perm
            )

            if not (user_qs.exists() or group_qs.exists()):
                raise serializers.ValidationError('User "{0}" have no permission with code "{1}"'.format(
                    user.username,
                    perm.code
                ))

        return attrs


class CheckLoginPassword(serializers.Serializer):
    username = serializers.CharField()
    password = serializers.CharField()

    def __init__(self, *args, request, **kwargs):
        super().__init__(*args, **kwargs)
        self.request = request

    def validate(self, attrs):
        username = attrs.get('username')
        password = attrs.get('password')
        if password and password:
            is_correct = bool(ModelBackend().authenticate(self.request, username=username, password=password))
            if not is_correct:
                raise serializers.ValidationError('Incorrect login and password combination.')
        return attrs


class User(serializers.ModelSerializer):
    class Meta:
        model = AuthUser
        exclude = ('password', )



# coding: utf-8
import requests
from django.conf import settings
from django.test import mock
from django.contrib.auth.models import User
from rest_framework import test, status
from rest_framework.exceptions import PermissionDenied
from .. import permissions


class PermissionTestCase(test.APITestCase):

    def setUp(self):
        self.request_factory = test.APIRequestFactory()
        self.request = self.request_factory.get('/some_url/')
        self.request.user = User.objects.create(username='tester')

    @mock.patch('sw_rest_auth.permissions.requests')
    def test_success_request(self, requests_mock):
        requests_mock.get.return_value.status_code = status.HTTP_200_OK

        perm_code = 'TEST_CODE'
        perm = permissions.CodePermission.decorate(perm_code)()
        has_permission = perm.has_permission(self.request, mock.Mock())
        self.assertTrue(has_permission)

        call_args, call_kwargs = requests_mock.get.call_args
        self.assertEqual(settings.AUTH_SERVICE_CHECK_PERM_URL, call_args[0])
        self.assertEqual(self.request.user.username, call_kwargs['params']['user'])
        self.assertEqual(perm_code, call_kwargs['params']['perm'])

    @mock.patch('sw_rest_auth.permissions.requests')
    def test_validation_error(self, requests_mock):
        requests_mock.get.return_value.status_code = status.HTTP_400_BAD_REQUEST

        perm_code = 'TEST_CODE'
        perm = permissions.CodePermission.decorate(perm_code)()

        exception = None
        try:
            perm.has_permission(self.request, mock.Mock())
        except PermissionDenied as ex:
            exception = ex
        self.assertTrue(exception)

    @mock.patch('sw_rest_auth.permissions.requests.head', side_effect=requests.ConnectionError)
    def test_connection_error(self, requests_mock):
        perm_code = 'TEST_CODE'
        perm = permissions.CodePermission.decorate(perm_code)()

        exception = None
        try:
            perm.has_permission(self.request, mock.Mock())
        except PermissionDenied as ex:
            exception = ex
        self.assertTrue(exception)



# coding: utf-8
from django.db import models
from django.contrib.auth.models import User, Group


class Permission(models.Model):
    code = models.CharField(max_length=50, unique=True, help_text='code for using in permission class in program code')
    name = models.CharField(max_length=100, unique=True, help_text='human permission name')
    description = models.TextField(blank=True, help_text='detailed description')

    class Meta:
        ordering = ('code', )

    def __str__(self):
        return '{0} ({1})'.format(self.name, self.code)


class UserPermission(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    permission = models.ForeignKey(Permission, on_delete=models.CASCADE)

    class Meta:
        unique_together = ('user', 'permission')
        ordering = ('user__username', 'permission__code')

    def __str__(self):
        return '{0} - {1}'.format(self.user, self.permission)


class GroupPermission(models.Model):
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    permission = models.ForeignKey(Permission, on_delete=models.CASCADE)

    class Meta:
        unique_together = ('group', 'permission')
        ordering = ('group__name', 'permission__code')

    def __str__(self):
        return '{0} - {1}'.format(self.group, self.permission)



ROOT_URLCONF = 'sw_rest_auth.urls'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'rest_framework.authtoken',
    'sw_rest_auth',
]

SECRET_KEY = "123"

REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.TokenAuthentication',
    ),
    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.JSONRenderer',
    ),
}

AUTH_SERVICE_CHECK_TOKEN_URL = 'https://auth-service/check_token/'
AUTH_SERVICE_CHECK_PERM_URL = 'https://auth-service/check_perm/'
AUTH_TOKEN = '321'



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-12-23 18:41
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0007_alter_validators_add_error_messages'),
        ('sw_rest_auth', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='GroupPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
                ('permission', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sw_rest_auth.Permission')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='grouppermission',
            unique_together=set([('group', 'permission')]),
        ),
    ]
